@@ -1,9 +1,12 @@
-/*! ========================================================================
-** Synfig
-** Template File
-** $Id: lyr_freetype.cpp,v 1.5 2005/01/24 05:00:18 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file lyr_freetype.cpp
+**	\brief Template Header
 **
+**	$Id: lyr_freetype.cpp 373 2007-03-23 16:02:09Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
+**	Copyright (c) 2006 Paul Wise
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -14,6 +17,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -29,6 +33,9 @@
 #ifdef HAVE_CONFIG_H
 #	include <config.h>
 #endif
+#ifdef WITH_FONTCONFIG
+#include <fontconfig/fontconfig.h>
+#endif
 
 #include "lyr_freetype.h"
 
@@ -58,13 +65,13 @@ SYNFIG_LAYER_SET_NAME(lyr_freetype,"text");
 SYNFIG_LAYER_SET_LOCAL_NAME(lyr_freetype,_("Simple Text"));
 SYNFIG_LAYER_SET_CATEGORY(lyr_freetype,_("Typography"));
 SYNFIG_LAYER_SET_VERSION(lyr_freetype,"0.2");
-SYNFIG_LAYER_SET_CVS_ID(lyr_freetype,"$Id: lyr_freetype.cpp,v 1.5 2005/01/24 05:00:18 darco Exp $");
+SYNFIG_LAYER_SET_CVS_ID(lyr_freetype,"$Id: lyr_freetype.cpp 373 2007-03-23 16:02:09Z dooglus $");
 
 /* === P R O C E D U R E S ================================================= */
 
 /*Glyph::~Glyph()
 {
-	if(glyph)FT_Done_Glyph(glyph);	
+	if(glyph)FT_Done_Glyph(glyph);
 }
 */
 void
@@ -84,7 +91,7 @@ TextLine::clear_and_free()
 lyr_freetype::lyr_freetype()
 {
 	face=0;
-	
+
 	size=Vector(0.25,0.25);
 	text=_("Text Layer");
 	color=Color::black();
@@ -100,9 +107,9 @@ lyr_freetype::lyr_freetype()
 	old_version=false;
 	set_blend_method(Color::BLEND_COMPOSITE);
 	needs_sync_=true;
-	
+
 	new_font(family,style,weight);
-	
+
 	invert=false;
 }
 
@@ -114,7 +121,7 @@ lyr_freetype::~lyr_freetype()
 
 void
 lyr_freetype::new_font(const synfig::String &family, int style, int weight)
-{		
+{
 	if(
 		!new_font_(family,style,weight) &&
 		!new_font_(family,style,WEIGHT_NORMAL) &&
@@ -134,7 +141,7 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 
 	if(new_face(font_fam_))
 		return true;
-	
+
 	//start evil hack
 	for(unsigned int i=0;i<font_fam.size();i++)font_fam[i]=tolower(font_fam[i]);
 	//end evil hack
@@ -146,7 +153,7 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 		else
 #endif
 		font_fam="sans serif";
-	
+
 	if(font_fam=="sans serif" || font_fam=="arial")
 	{
 		String arial("arial");
@@ -203,7 +210,7 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 		if(new_face(filename))
 			return true;
 	}
-	
+
 	if(font_fam=="trebuchet")
 	{
 		String filename("trebuc");
@@ -219,8 +226,8 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 		if(new_face(filename))
 			return true;
 	}
-		
-	
+
+
 	if(font_fam=="sans serif" || font_fam=="luxi sans")
 	{
 		{
@@ -231,8 +238,8 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 				luxi+='r';
 			if(style==PANGO_STYLE_ITALIC||style==PANGO_STYLE_OBLIQUE)
 				luxi+='i';
-			
-			
+
+
 			if(new_face(luxi))
 				return true;
 		}
@@ -251,7 +258,7 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 				luxi+='r';
 			if(style==PANGO_STYLE_ITALIC||style==PANGO_STYLE_OBLIQUE)
 				luxi+='i';
-			
+
 			if(new_face(luxi))
 				return true;
 		}
@@ -270,7 +277,7 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 				luxi+='r';
 			if(style==PANGO_STYLE_ITALIC||style==PANGO_STYLE_OBLIQUE)
 				luxi+='i';
-			
+
 			if(new_face(luxi))
 				return true;
 		}
@@ -280,9 +287,9 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 		if(new_face("Times"))
 			return true;
 	}
-	
+
 	return new_face(font_fam_) || new_face(font_fam);
-	
+
 	return false;
 }
 
@@ -290,21 +297,21 @@ lyr_freetype::new_font_(const synfig::String &font_fam_, int style, int weight)
 void fss2path(char *path, FSSpec *fss)
 {
   int l;             //fss->name contains name of last item in path
-  for(l=0; l<(fss->name[0]); l++) path[l] = fss->name[l + 1]; 
+  for(l=0; l<(fss->name[0]); l++) path[l] = fss->name[l + 1];
   path[l] = 0;
 
   if(fss->parID != fsRtParID) //path is more than just a volume name
-  { 
+  {
     int i, len;
     CInfoPBRec pb;
-    
+
     pb.dirInfo.ioNamePtr = fss->name;
     pb.dirInfo.ioVRefNum = fss->vRefNum;
     pb.dirInfo.ioDrParID = fss->parID;
     do
     {
       pb.dirInfo.ioFDirIndex = -1;  //get parent directory name
-      pb.dirInfo.ioDrDirID = pb.dirInfo.ioDrParID;   
+      pb.dirInfo.ioDrDirID = pb.dirInfo.ioDrParID;
       if(PBGetCatInfoSync(&pb) != noErr) break;
 
       len = fss->name[0] + 1;
@@ -352,7 +359,7 @@ lyr_freetype::new_face(const String &newfont)
 			char filename[512];
 			fss2path(filename,&fs_spec);
 			//FSSpecToNativePathName(fs_spec,filename,sizeof(filename)-1, 0);
-			
+
 			error=FT_New_Face(ft_library, filename, face_index,&face);
 			//error=FT_New_Face_From_FSSpec(ft_library, &fs_spec, face_index,&face);
 			synfig::info(__FILE__":%d: \"%s\" (%s) -- ft_error=%d",__LINE__,newfont.c_str(),filename,error);
@@ -362,7 +369,38 @@ lyr_freetype::new_face(const String &newfont)
 			synfig::info(__FILE__":%d: \"%s\" -- ft_error=%d",__LINE__,newfont.c_str(),error);
 			// Unable to generate fs_spec
 		}
-		  
+
+	}
+#endif
+
+#ifdef WITH_FONTCONFIG
+	if(error)
+	{
+		FcFontSet *fs;
+		FcResult result;
+		if( !FcInit() )
+		{
+			synfig::warning("lyr_freetype: fontconfig: %s",_("unable to initialise"));
+			error = 1;
+		} else {
+			FcPattern* pat = FcNameParse((FcChar8 *) newfont.c_str());
+			FcConfigSubstitute(0, pat, FcMatchPattern);
+			FcDefaultSubstitute(pat);
+			FcPattern *match;
+			fs = FcFontSetCreate();
+			match = FcFontMatch(0, pat, &result);
+			if (match)
+				FcFontSetAdd(fs, match);
+			if (pat)
+				FcPatternDestroy(pat);
+			if(fs){
+				FcChar8* file;
+				if( FcPatternGetString (fs->fonts[0], FC_FILE, 0, &file) == FcResultMatch )
+					error=FT_New_Face(ft_library,(const char*)file,face_index,&face);
+				FcFontSetDestroy(fs);
+			} else
+				synfig::warning("lyr_freetype: fontconfig: %s",_("empty font set"));
+		}
 	}
 #endif
 
@@ -431,7 +469,7 @@ lyr_freetype::set_param(const String & param, const ValueBase &value)
 	IMPORT_PLUS(vcompress,needs_sync_=true);
 	IMPORT_PLUS(use_kerning,needs_sync_=true);
 	IMPORT_PLUS(grid_fit,needs_sync_=true);
-	
+
 	return Layer_Composite::set_param(param,value);
 }
 
@@ -452,7 +490,7 @@ lyr_freetype::get_param(const String& param)const
 	EXPORT(use_kerning);
 	EXPORT(grid_fit);
 	EXPORT(invert);
-	
+
 	EXPORT_NAME();
 	EXPORT_VERSION();
 
@@ -554,10 +592,10 @@ void
 lyr_freetype::sync()
 {
 	needs_sync_=false;
-	
-	
-	
-	
+
+
+
+
 }
 
 Color
@@ -565,7 +603,7 @@ lyr_freetype::get_color(Context context, const synfig::Point &pos)const
 {
 	if(needs_sync_)
 		const_cast<lyr_freetype*>(this)->sync();
-	
+
 	if(!face)
 		return context.get_color(pos);
 	return context.get_color(pos);
@@ -579,18 +617,18 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 	if(needs_sync_)
 		const_cast<lyr_freetype*>(this)->sync();
 
-	
-	
-	
+
+
+
 	int error;
 	Vector size(lyr_freetype::size*2);
-	
+
 	if(!context.accelerated_render(surface,quality,renddesc,cb))
 		return false;
-	
+
 	if(is_disabled() || text.empty())
 		return true;
-	
+
 	// If there is no font loaded, just bail
 	if(!face)
 	{
@@ -603,7 +641,7 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 	{
 		text=basename(get_canvas()->get_file_name());
 	}
-	
+
 	// Width and Height of a pixel
 	Vector::value_type pw=renddesc.get_w()/(renddesc.get_br()[0]-renddesc.get_tl()[0]);
 	Vector::value_type ph=renddesc.get_h()/(renddesc.get_br()[1]-renddesc.get_tl()[1]);
@@ -616,7 +654,7 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
     //int by=(int)((pos[1]-renddesc.get_tl()[1])*ph*64+0.5);
     int bx=0;
     int by=0;
-	
+
     // If the font is the size of a pixel, don't bother rendering any text
 	if(w<=1 || h<=1)
 	{
@@ -628,11 +666,11 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 
 #define CHAR_RESOLUTION		(64)
 	error = FT_Set_Char_Size(
-		face,						// handle to face object           
-		(int)CHAR_RESOLUTION,	// char_width in 1/64th of points 
-		(int)CHAR_RESOLUTION,	// char_height in 1/64th of points 
-		round_to_int(abs(size[0]*pw*CHAR_RESOLUTION)),						// horizontal device resolution    
-		round_to_int(abs(size[1]*ph*CHAR_RESOLUTION)) );						// vertical device resolution      
+		face,						// handle to face object
+		(int)CHAR_RESOLUTION,	// char_width in 1/64th of points
+		(int)CHAR_RESOLUTION,	// char_height in 1/64th of points
+		round_to_int(abs(size[0]*pw*CHAR_RESOLUTION)),						// horizontal device resolution
+		round_to_int(abs(size[1]*ph*CHAR_RESOLUTION)) );						// vertical device resolution
 
 	// Here is where we can compensate for the
 	// error in freetype's rendering engine.
@@ -644,8 +682,7 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 
 	if(error)
 	{
-		if(cb)cb->error(string("lyr_freetype:")+_("Unable to set face size.")+strprintf(" (err=%d)",error));
-		return false;
+		if(cb)cb->warning(string("lyr_freetype:")+_("Unable to set face size.")+strprintf(" (err=%d)",error));
 	}
 
 	FT_GlyphSlot  slot = face->glyph;  // a small shortcut
@@ -687,7 +724,7 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 
 			if(grid_fit)
 				FT_Get_Kerning( face, previous, glyph_index, ft_kerning_default, &delta );
-			else	
+			else
 				FT_Get_Kerning( face, previous, glyph_index, ft_kerning_unfitted, &delta );
 
 			if(compress<1.0f)
@@ -701,9 +738,9 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 				by += delta.y;
 			}
         }
-		
+
 		Glyph curr_glyph;
-		
+
         // store current pen position
         curr_glyph.pos.x = bx;
         curr_glyph.pos.y = by;
@@ -721,10 +758,10 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 
         // record current glyph index
         previous = glyph_index;
-		
+
 		// Update the line width
 		lines.front().width=bx+slot->advance.x;
-		
+
 		// increment pen position
 		if(multiplier>1)
 			bx += round_to_int(slot->advance.x*multiplier*compress)-bx%round_to_int(slot->advance.x*multiplier*compress);
@@ -736,9 +773,9 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 		by += slot->advance.y*multiplier;
 
 		lines.front().glyph_table.push_back(curr_glyph);
-		
+
 	}
-      
+
 
 	//float	string_height;
 	//string_height=(((lines.size()-1)*face->size->metrics.height+lines.back().actual_height()));
@@ -746,7 +783,7 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 	//int string_height=face->size->metrics.ascender;
 //#define METRICS_SCALE_ONE		(65536.0f)
 #define METRICS_SCALE_ONE		((float)(1<<16))
-	
+
 	float line_height;
 	line_height=vcompress*((float)face->height*(((float)face->size->metrics.y_scale/METRICS_SCALE_ONE)));
 
@@ -754,26 +791,26 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 	string_height=round_to_int(((lines.size()-1)*line_height+lines.back().actual_height()));
 	//synfig::info("string_height=%d",string_height);
 	//synfig::info("line_height=%f",line_height);
-	
+
 	/*
  --	** -- RENDER THE GLYPHS ---------------------------------------------------
 	*/
 
 	Surface src_;
 	Surface *src_surface;
-	
+
 	src_surface=surface;
 
 	if(invert)
 	{
 		src_=*surface;
 		Surface::alpha_pen pen(surface->begin(),get_amount(),get_blend_method());
-		
+
 		surface->fill(color,pen,src_.get_w(),src_.get_h());
-		
+
 		src_surface=&src_;
 	}
-	
+
 	{
 	std::list<TextLine>::iterator iter;
 	int curr_line;
@@ -783,22 +820,22 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 		by=round_to_int((pos[1]-renddesc.get_tl()[1])*ph*CHAR_RESOLUTION+(1.0-orient[1])*string_height-line_height*curr_line);
 		//by=round_to_int(vcompress*((pos[1]-renddesc.get_tl()[1])*ph*64+(1.0-orient[1])*string_height-face->size->metrics.height*curr_line));
 		//synfig::info("curr_line=%d, bx=%d, by=%d",curr_line,bx,by);
-		
+
 		std::vector<Glyph>::iterator iter2;
 		for(iter2=iter->glyph_table.begin();iter2!=iter->glyph_table.end();++iter2)
 		{
 			FT_Glyph  image(iter2->glyph);
 			FT_Vector pen;
 			FT_BitmapGlyph  bit;
-				
+
 			pen.x = bx + iter2->pos.x;
 			pen.y = by + iter2->pos.y;
-			
+
 			//synfig::info("GLYPH: pen.x=%d, pen,y=%d",curr_line,(pen.x+32)>>6,(pen.y+32)>>6);
-		
+
 			error = FT_Glyph_To_Bitmap( &image, ft_render_mode_normal,0/*&pen*/, 1 );
 			if(error) { FT_Done_Glyph( image ); continue; }
-		
+
 			bit = (FT_BitmapGlyph)image;
 
 			for(v=0;v<bit->bitmap.rows;v++)
@@ -817,13 +854,13 @@ lyr_freetype::accelerated_render(Context context,Surface *surface,int quality, c
 						(*surface)[y][x]=Color::blend(color,(*src_surface)[y][x],myamount*get_amount(),get_blend_method());
 					}
 				}
-		
+
 			FT_Done_Glyph( image );
 		}
 		//iter->clear_and_free();
 	}
 	}
-   
+
 
 	return true;
 }
