@@ -2,7 +2,7 @@
 /*!	\file import.h
 **	\brief Template Header
 **
-**	$Id: import.h,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $
+**	$Id: import.h 335 2007-03-16 00:39:09Z dooglus $
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
@@ -53,10 +53,10 @@ protected:
 
 public:
 	~Import();
-	
+
 	virtual bool set_param(const synfig::String & param, const synfig::ValueBase &value);
 
-	virtual synfig::ValueBase get_param(const synfig::String & param)const;	
+	virtual synfig::ValueBase get_param(const synfig::String & param)const;
 
 	virtual Vocab get_param_vocab()const;
 
@@ -64,7 +64,7 @@ public:
 
 	virtual void set_time(synfig::Context context, synfig::Time time)const;
 
-	virtual void set_time(synfig::Context context, synfig::Time time, const synfig::Point &pos)const;
+	virtual void set_time(synfig::Context context, synfig::Time time, const synfig::Point &point)const;
 };
 
 /* === E N D =============================================================== */
