@@ -1,8 +1,10 @@
-/*! ========================================================================
-** Synfig
-** Image Layer_TimeLoop Layer Implementation
-** $Id: timeloop.cpp,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file timeloop.cpp
+**	\brief Image Layer_TimeLoop Layer Implementation
 **
+**	$Id: timeloop.cpp 373 2007-03-23 16:02:09Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
 **	This package is free software; you can redistribute it and/or
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -50,7 +53,7 @@ SYNFIG_LAYER_SET_NAME(Layer_TimeLoop,"timeloop");
 SYNFIG_LAYER_SET_LOCAL_NAME(Layer_TimeLoop,_("TimeLoop"));
 SYNFIG_LAYER_SET_CATEGORY(Layer_TimeLoop,_("Other"));
 SYNFIG_LAYER_SET_VERSION(Layer_TimeLoop,"0.1");
-SYNFIG_LAYER_SET_CVS_ID(Layer_TimeLoop,"$Id: timeloop.cpp,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $");
+SYNFIG_LAYER_SET_CVS_ID(Layer_TimeLoop,"$Id: timeloop.cpp 373 2007-03-23 16:02:09Z dooglus $");
 
 /* === P R O C E D U R E S ================================================= */
 
@@ -89,7 +92,7 @@ Layer::Vocab
 Layer_TimeLoop::get_param_vocab()const
 {
 	Layer::Vocab ret(Layer::get_param_vocab());
-	
+
 	ret.push_back(ParamDesc("start_time")
 		.set_local_name(_("Start Time"))
 	);
@@ -97,7 +100,7 @@ Layer_TimeLoop::get_param_vocab()const
 	ret.push_back(ParamDesc("end_time")
 		.set_local_name(_("End Time"))
 	);
-	
+
 	return ret;
 }
 
