@@ -1,8 +1,10 @@
-/*! ========================================================================
-** Synfig
-** bmp Target Module
-** $Id: mptr_bmp.cpp,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file mptr_bmp.cpp
+**	\brief bmp Target Module
 **
+**	$Id: mptr_bmp.cpp 373 2007-03-23 16:02:09Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
 **	This package is free software; you can redistribute it and/or
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -47,10 +50,10 @@ using namespace etl;
 /* === G L O B A L S ======================================================= */
 
 SYNFIG_IMPORTER_INIT(bmp_mptr);
-SYNFIG_IMPORTER_SET_NAME(bmp_mptr,"bmp_mptr");
+SYNFIG_IMPORTER_SET_NAME(bmp_mptr,"bmp");
 SYNFIG_IMPORTER_SET_EXT(bmp_mptr,"bmp");
 SYNFIG_IMPORTER_SET_VERSION(bmp_mptr,"0.1");
-SYNFIG_IMPORTER_SET_CVS_ID(bmp_mptr,"$Id: mptr_bmp.cpp,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $");
+SYNFIG_IMPORTER_SET_CVS_ID(bmp_mptr,"$Id: mptr_bmp.cpp 373 2007-03-23 16:02:09Z dooglus $");
 
 /* === M E T H O D S ======================================================= */
 
@@ -119,12 +122,12 @@ bmp_mptr::~bmp_mptr()
 }
 
 bool
-bmp_mptr::get_frame(synfig::Surface &surface,Time, synfig::ProgressCallback *cb)
+bmp_mptr::get_frame(synfig::Surface &surface,Time time, synfig::ProgressCallback *cb)
 {
 	FILE *file=fopen(filename.c_str(),"rb");
 	if(!file)
 	{
-		if(cb)cb->error("bmp_mptr::GetFrame(): "+strprintf(_("Unable to open %s"),filename.c_str())); 
+		if(cb)cb->error("bmp_mptr::GetFrame(): "+strprintf(_("Unable to open %s"),filename.c_str()));
 		else synfig::error("bmp_mptr::GetFrame(): "+strprintf(_("Unable to open %s"),filename.c_str()));
 		return false;
 	}
@@ -133,72 +136,72 @@ bmp_mptr::get_frame(synfig::Surface &surface,Time, synfig::ProgressCallback *cb)
 	BITMAPINFOHEADER infoheader;
 	char b_char=fgetc(file);
 	char m_char=fgetc(file);
-	
+
 	if(b_char!='B' || m_char!='M')
 	{
-		if(cb)cb->error("bmp_mptr::GetFrame(): "+strprintf(_("%s is not in BMP format"),filename.c_str())); 
+		if(cb)cb->error("bmp_mptr::GetFrame(): "+strprintf(_("%s is not in BMP format"),filename.c_str()));
 		else synfig::error("bmp_mptr::GetFrame(): "+strprintf(_("%s is not in BMP format"),filename.c_str()));
 		return false;
 	}
-		
+
 	if(fread(&fileheader.bfSize, 1, sizeof(BITMAPFILEHEADER)-4, file)!=sizeof(BITMAPFILEHEADER)-4)
 	{
-		String str("bmp_mptr::get_frame(): "+strprintf(_("Failure while reading BITMAPFILEHEADER from %s"),filename.c_str())); 
-		if(cb)cb->error(str); 
+		String str("bmp_mptr::get_frame(): "+strprintf(_("Failure while reading BITMAPFILEHEADER from %s"),filename.c_str()));
+		if(cb)cb->error(str);
 		else synfig::error(str);
 		return false;
 	}
-		
+
 	if(fread(&infoheader, 1, sizeof(BITMAPINFOHEADER), file)!=sizeof(BITMAPINFOHEADER))
 	{
-		String str("bmp_mptr::get_frame(): "+strprintf(_("Failure while reading BITMAPINFOHEADER from %s"),filename.c_str())); 
-		if(cb)cb->error(str); 
+		String str("bmp_mptr::get_frame(): "+strprintf(_("Failure while reading BITMAPINFOHEADER from %s"),filename.c_str()));
+		if(cb)cb->error(str);
 		else synfig::error(str);
 		return false;
 	}
-	
+
 	int offset=little_endian(fileheader.bfOffsetBits);
-	
-	if(offset!=sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)-2)	
+
+	if(offset!=sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)-2)
 	{
-		String str("bmp_mptr::get_frame(): "+strprintf(_("Bad BITMAPFILEHEADER in %s. (bfOffsetBits=%d, should be %d)"),filename.c_str(),offset,sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)-2)); 
-		if(cb)cb->error(str); 
+		String str("bmp_mptr::get_frame(): "+strprintf(_("Bad BITMAPFILEHEADER in %s. (bfOffsetBits=%d, should be %d)"),filename.c_str(),offset,sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)-2));
+		if(cb)cb->error(str);
 		else synfig::error(str);
 		return false;
 	}
 
 	if(little_endian(infoheader.biSize)!=little_endian(40))
 	{
-		String str("bmp_mptr::get_frame(): "+strprintf(_("Bad BITMAPINFOHEADER in %s. (biSize=%d, should be 40)"),filename.c_str(),little_endian(infoheader.biSize))); 
-		if(cb)cb->error(str); 
+		String str("bmp_mptr::get_frame(): "+strprintf(_("Bad BITMAPINFOHEADER in %s. (biSize=%d, should be 40)"),filename.c_str(),little_endian(infoheader.biSize)));
+		if(cb)cb->error(str);
 		else synfig::error(str);
 		return false;
 	}
-	
+
 	int w,h,bit_count;
 
 	w=little_endian(infoheader.biWidth);
 	h=little_endian(infoheader.biHeight);
 	bit_count=little_endian_short(infoheader.biBitCount);
-	
+
 	synfig::warning("w:%d\n",w);
 	synfig::warning("h:%d\n",h);
 	synfig::warning("bit_count:%d\n",bit_count);
-	
+
 	if(little_endian(infoheader.biCompression))
 	{
-		if(cb)cb->error("bmp_mptr::GetFrame(): "+string(_("Reading compressed bitmaps is not supported"))); 
+		if(cb)cb->error("bmp_mptr::GetFrame(): "+string(_("Reading compressed bitmaps is not supported")));
 		else synfig::error("bmp_mptr::GetFrame(): "+string(_("Reading compressed bitmaps is not supported")));
 		return false;
 	}
 
 	if(bit_count!=24 && bit_count!=32)
 	{
-		if(cb)cb->error("bmp_mptr::GetFrame(): "+strprintf(_("Unsupported bit depth (bit_count=%d, should be 24 or 32)"),bit_count)); 
+		if(cb)cb->error("bmp_mptr::GetFrame(): "+strprintf(_("Unsupported bit depth (bit_count=%d, should be 24 or 32)"),bit_count));
 		else synfig::error("bmp_mptr::GetFrame(): "+strprintf(_("Unsupported bit depth (bit_count=%d, should be 24 or 32)"),bit_count));
 		return false;
 	}
-	
+
 	int x;
 	int y;
 	surface.set_wh(w,h);
@@ -211,7 +214,7 @@ bmp_mptr::get_frame(synfig::Surface &surface,Time, synfig::ProgressCallback *cb)
 			float b=gamma().b_U8_to_F32((unsigned char)fgetc(file));
 			float g=gamma().g_U8_to_F32((unsigned char)fgetc(file));
 			float r=gamma().r_U8_to_F32((unsigned char)fgetc(file));
-			
+
 			surface[h-y-1][x]=Color(
 				r,
 				g,
@@ -221,8 +224,9 @@ bmp_mptr::get_frame(synfig::Surface &surface,Time, synfig::ProgressCallback *cb)
 			if(bit_count==32)
 				fgetc(file);
 		}
-	
 
-	fclose(file);	
+
+	fclose(file);
 	return true;
 }
+ 
