@@ -1,8 +1,10 @@
-/*! ========================================================================
-** Synfig
-** Template File
-** $Id: curvegradient.cpp,v 1.2 2005/01/13 06:48:39 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file curvegradient.cpp
+**	\brief Template Header
 **
+**	$Id: curvegradient.cpp 472 2007-04-18 09:14:33Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
 **	This package is free software; you can redistribute it and/or
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -53,7 +56,7 @@ SYNFIG_LAYER_SET_NAME(CurveGradient,"curve_gradient");
 SYNFIG_LAYER_SET_LOCAL_NAME(CurveGradient,_("Curve Gradient"));
 SYNFIG_LAYER_SET_CATEGORY(CurveGradient,_("Gradients"));
 SYNFIG_LAYER_SET_VERSION(CurveGradient,"0.0");
-SYNFIG_LAYER_SET_CVS_ID(CurveGradient,"$Id: curvegradient.cpp,v 1.2 2005/01/13 06:48:39 darco Exp $");
+SYNFIG_LAYER_SET_CVS_ID(CurveGradient,"$Id: curvegradient.cpp 472 2007-04-18 09:14:33Z dooglus $");
 
 /* === P R O C E D U R E S ================================================= */
 
@@ -73,16 +76,18 @@ inline float calculate_distance(const std::vector<synfig::BLinePoint>& bline)
 {
 	std::vector<synfig::BLinePoint>::const_iterator iter,next,ret;
 	std::vector<synfig::BLinePoint>::const_iterator end(bline.end());
-	
+
 	float dist(0);
-	
+
+	if (bline.empty()) return dist;
+
 	next=bline.begin();
-	
+
 	//if(loop)
 	//	iter=--bline.end();
 	//else
-		iter=next++;
-	
+	iter=next++;
+
 	for(;next!=end;iter=next++)
 	{
 		// Setup the curve
@@ -90,8 +95,7 @@ inline float calculate_distance(const std::vector<synfig::BLinePoint>& bline)
 			iter->get_vertex(),
 			next->get_vertex(),
 			iter->get_tangent2(),
-			next->get_tangent1()
-		);
+			next->get_tangent1());
 
 //		dist+=calculate_distance(*iter,*next);
 		dist+=curve.length();
@@ -101,28 +105,29 @@ inline float calculate_distance(const std::vector<synfig::BLinePoint>& bline)
 }
 
 std::vector<synfig::BLinePoint>::const_iterator
-find_closest(const std::vector<synfig::BLinePoint>& bline,const Point& p,bool loop=false,float *bline_dist_ret=0)
+find_closest(bool fast, const std::vector<synfig::BLinePoint>& bline,const Point& p,float& t,bool loop=false,float *bline_dist_ret=0)
 {
 	std::vector<synfig::BLinePoint>::const_iterator iter,next,ret;
 	std::vector<synfig::BLinePoint>::const_iterator end(bline.end());
-	
+
 	ret=bline.end();
 	float dist(100000000000.0);
-	
+
 	next=bline.begin();
 
 	float best_bline_dist(0);
 	float best_bline_len(0);
 	float total_bline_dist(0);
+	float best_pos(0);
 	etl::hermite<Vector> best_curve;
-	
+
 	if(loop)
 		iter=--bline.end();
 	else
 		iter=next++;
 
 	Point bp;
-	
+
 	for(;next!=end;iter=next++)
 	{
 		// Setup the curve
@@ -130,12 +135,11 @@ find_closest(const std::vector<synfig::BLinePoint>& bline,const Point& p,bool lo
 			iter->get_vertex(),
 			next->get_vertex(),
 			iter->get_tangent2(),
-			next->get_tangent1()
-		);
+			next->get_tangent1());
 
 		/*
 		const float t(curve.find_closest(p,6,0.01,0.99));
-		bp=curve(t);if((bp-p).mag_squared()<dist) { ret=iter; dist=(bp-p).mag_squared(); ret_t=t; }		
+		bp=curve(t);if((bp-p).mag_squared()<dist) { ret=iter; dist=(bp-p).mag_squared(); ret_t=t; }
 		*/
 
 		float thisdist(0);
@@ -145,26 +149,46 @@ find_closest(const std::vector<synfig::BLinePoint>& bline,const Point& p,bool lo
 			//len=calculate_distance(*iter,*next);
 			len=curve.length();
 		}
-		
-#define POINT_CHECK(x) bp=curve(x);	thisdist=(bp-p).mag_squared(); if(thisdist<dist) { ret=iter; dist=thisdist; best_bline_dist=total_bline_dist; best_bline_len=len; best_curve=curve; }
 
-		POINT_CHECK(0.0001);
-		POINT_CHECK((1.0/6.0));
-		POINT_CHECK((2.0/6.0));
-		POINT_CHECK((3.0/6.0));
-		POINT_CHECK((4.0/6.0));
-		POINT_CHECK((5.0/6.0));
-		POINT_CHECK(0.9999);
+		if (fast)
+		{
+#define POINT_CHECK(x) bp=curve(x);	thisdist=(bp-p).mag_squared(); if(thisdist<dist) { ret=iter; dist=thisdist; best_bline_dist=total_bline_dist; best_bline_len=len; best_curve=curve; }
+			POINT_CHECK(0.0001);
+			POINT_CHECK((1.0/6.0));
+			POINT_CHECK((2.0/6.0));
+			POINT_CHECK((3.0/6.0));
+			POINT_CHECK((4.0/6.0));
+			POINT_CHECK((5.0/6.0));
+			POINT_CHECK(0.9999);
+		}
+		else
+		{
+			float pos = curve.find_closest(fast, p);
+			thisdist=(curve(pos)-p).mag_squared();
+			if(thisdist<dist)
+			{
+				ret=iter;
+				dist=thisdist;
+				best_bline_dist=total_bline_dist;
+				best_bline_len=len;
+				best_curve=curve;
+				best_pos = pos;
+			}
+		}
 
 		total_bline_dist+=len;
 	}
 
+	t = best_pos;
+
 	if(bline_dist_ret)
 	{
-		*bline_dist_ret=best_bline_dist+best_curve.find_distance(0,best_curve.find_closest(p));
-//		*bline_dist_ret=best_bline_dist+best_curve.find_closest(p)*best_bline_len;
+		//! \todo is this a redundant call to find_closest()?
+		// note bline_dist_ret is null except when 'perpendicular' is true
+		*bline_dist_ret=best_bline_dist+best_curve.find_distance(0,best_curve.find_closest(fast, p));
+//		*bline_dist_ret=best_bline_dist+best_curve.find_closest(fast, p)*best_bline_len;
 	}
-	
+
 	return ret;
 }
 
@@ -173,38 +197,33 @@ find_closest(const std::vector<synfig::BLinePoint>& bline,const Point& p,bool lo
 inline void
 CurveGradient::sync()
 {
-	diff=(p2-p1);
-	const Real mag(diff.inv_mag());
-	diff*=mag*mag;
-
 	curve_length_=calculate_distance(bline);
 }
 
 
 CurveGradient::CurveGradient():
-	p1(1,1),
-	p2(-1,-1),
 	offset(0,0),
 	width(0.25),
 	gradient(Color::black(), Color::white()),
 	loop(false),
 	zigzag(false),
-	perpendicular(false)
+	perpendicular(false),
+	fast(true)
 {
 	bline.push_back(BLinePoint());
 	bline.push_back(BLinePoint());
 	bline.push_back(BLinePoint());
-	bline[0].set_vertex(Point(0,1));	
-	bline[1].set_vertex(Point(0,-1));	
+	bline[0].set_vertex(Point(0,1));
+	bline[1].set_vertex(Point(0,-1));
 	bline[2].set_vertex(Point(1,0));
-	bline[0].set_tangent(bline[1].get_vertex()-bline[2].get_vertex()*0.5f);	
-	bline[1].set_tangent(bline[2].get_vertex()-bline[0].get_vertex()*0.5f);	
-	bline[2].set_tangent(bline[0].get_vertex()-bline[1].get_vertex()*0.5f);	
-	bline[0].set_width(1.0f);	
-	bline[1].set_width(1.0f);	
-	bline[2].set_width(1.0f);	
+	bline[0].set_tangent(bline[1].get_vertex()-bline[2].get_vertex()*0.5f);
+	bline[1].set_tangent(bline[2].get_vertex()-bline[0].get_vertex()*0.5f);
+	bline[2].set_tangent(bline[0].get_vertex()-bline[1].get_vertex()*0.5f);
+	bline[0].set_width(1.0f);
+	bline[1].set_width(1.0f);
+	bline[2].set_width(1.0f);
 	bline_loop=true;
-	
+
 	sync();
 }
 
@@ -218,7 +237,7 @@ CurveGradient::color_func(const Point &point_, int quality, float supersample)co
 	Real dist;
 
 	float perp_dist;
-	
+
 	if(bline.size()==0)
 		return Color::alpha();
 	else if(bline.size()==1)
@@ -229,84 +248,86 @@ CurveGradient::color_func(const Point &point_, int quality, float supersample)co
 	}
 	else
 	{
+		float t;
 		Point point(point_-offset);
-		
+
 		std::vector<synfig::BLinePoint>::const_iterator iter,next;
-		
+
 		// Figure out the BLinePoints we will be using,
 		// Taking into account looping.
 		if(perpendicular)
 		{
-			next=find_closest(bline,point,bline_loop,&perp_dist);
+			next=find_closest(fast,bline,point,t,bline_loop,&perp_dist);
 			perp_dist/=curve_length_;
 		}
 		else
 		{
-			next=find_closest(bline,point,bline_loop);
+			next=find_closest(fast,bline,point,t,bline_loop);
 		}
-			iter=next++;
-			if(next==bline.end()) next=bline.begin();
-			
-			// Setup the curve
-			etl::hermite<Vector> curve(
-				iter->get_vertex(),
-				next->get_vertex(),
-				iter->get_tangent2(),
-				next->get_tangent1()
+
+		iter=next++;
+		if(next==bline.end()) next=bline.begin();
+
+		// Setup the curve
+		etl::hermite<Vector> curve(
+			iter->get_vertex(),
+			next->get_vertex(),
+			iter->get_tangent2(),
+			next->get_tangent1()
 			);
-			
-			// Setup the derivative function
-			etl::derivative<etl::hermite<Vector> > deriv(curve);
-	
-			int search_iterations(7);
-			
-			/*if(quality==0)search_iterations=8;
-			else if(quality<=2)search_iterations=10;
-			else if(quality<=4)search_iterations=8;
-			*/
-			if(!perpendicular)
-			{
-				if(quality<=6)search_iterations=7;
-				else if(quality<=7)search_iterations=6;
-				else if(quality<=8)search_iterations=5;
-				else search_iterations=4;
-			}
-			else
-			{
-				if(quality>7)
-					search_iterations=4;
-			}
-			
-			// Figure out the closest point on the curve
-			const float t(curve.find_closest(point,search_iterations));
-				
-			
-			// Calculate our values
-			p1=curve(t);
-			tangent=deriv(t).norm();
-			
-			if(perpendicular)
-			{
-				tangent*=curve_length_;
-				p1-=tangent*perp_dist;
-				tangent=-tangent.perp();
-			}
-			else
-			{
-				thickness=(next->get_width()-iter->get_width())*t+iter->get_width();
-			}
-		//}		
+
+		// Setup the derivative function
+		etl::derivative<etl::hermite<Vector> > deriv(curve);
+
+		int search_iterations(7);
+
+		/*if(quality==0)search_iterations=8;
+		  else if(quality<=2)search_iterations=10;
+		  else if(quality<=4)search_iterations=8;
+		*/
+		if(!perpendicular)
+		{
+			if(quality<=6)search_iterations=7;
+			else if(quality<=7)search_iterations=6;
+			else if(quality<=8)search_iterations=5;
+			else search_iterations=4;
+		}
+		else
+		{
+			if(quality>7)
+				search_iterations=4;
+		}
+
+		// Figure out the closest point on the curve
+		if (fast)
+			t = curve.find_closest(fast, point,search_iterations);
+
+
+		// Calculate our values
+		p1=curve(t);
+		tangent=deriv(t).norm();
+
+		if(perpendicular)
+		{
+			tangent*=curve_length_;
+			p1-=tangent*perp_dist;
+			tangent=-tangent.perp();
+		}
+		else
+		{
+			thickness=(next->get_width()-iter->get_width())*t+iter->get_width();
+		}
+		//}
 	}
-	
 
 	if(!perpendicular)
 	{
-			diff=tangent.perp()*thickness*width;
-			p1-=diff*0.5;
-			const Real mag(diff.inv_mag());
-			supersample=supersample*mag;
-			diff*=mag*mag;
-			dist=((point_-offset)*diff-p1*diff);
+		diff=tangent.perp()*thickness*width;
+		p1-=diff*0.5;
+		const Real mag(diff.inv_mag());
+		supersample=supersample*mag;
+		diff*=mag*mag;
+		dist=((point_-offset)*diff-p1*diff);
 	}
 	else
 	{
@@ -321,18 +342,18 @@ CurveGradient::color_func(const Point &point_, int quality, float supersample)co
 		}
 		else
 		{
-		diff=tangent.perp();
-		//p1-=diff*0.5;
-		const Real mag(diff.inv_mag());
-		supersample=supersample*mag;
-		diff*=mag*mag;
-		dist=((point_-offset)*diff-p1*diff);
+			diff=tangent.perp();
+			//p1-=diff*0.5;
+			const Real mag(diff.inv_mag());
+			supersample=supersample*mag;
+			diff*=mag*mag;
+			dist=((point_-offset)*diff-p1*diff);
 		}
 	}
-	
+
 	if(loop)
 		dist-=floor(dist);
-	
+
 	if(zigzag)
 	{
 		dist*=2.0;
@@ -344,14 +365,20 @@ CurveGradient::color_func(const Point &point_, int quality, float supersample)co
 	{
 		if(dist+supersample*0.5>1.0)
 		{
-			Color pool(gradient(dist,supersample*0.5).premult_alpha()*(1.0-(dist-supersample*0.5)));
-			pool+=gradient((dist+supersample*0.5)-1.0,supersample*0.5).premult_alpha()*((dist+supersample*0.5)-1.0);
+			float  left(supersample*0.5-(dist-1.0));
+			float right(supersample*0.5+(dist-1.0));
+			Color pool(gradient(1.0-(left*0.5),left).premult_alpha()*left/supersample);
+			if (zigzag) pool+=gradient(1.0-right*0.5,right).premult_alpha()*right/supersample;
+			else		pool+=gradient(right*0.5,right).premult_alpha()*right/supersample;
 			return pool.demult_alpha();
 		}
 		if(dist-supersample*0.5<0.0)
 		{
-			Color pool(gradient(dist,supersample*0.5).premult_alpha()*(dist+supersample*0.5));
-			pool+=gradient(1.0-(dist-supersample*0.5),supersample*0.5).premult_alpha()*(-(dist-supersample*0.5));
+			float  left(supersample*0.5-dist);
+			float right(supersample*0.5+dist);
+			Color pool(gradient(right*0.5,right).premult_alpha()*right/supersample);
+			if (zigzag) pool+=gradient(left*0.5,left).premult_alpha()*left/supersample;
+			else		pool+=gradient(1.0-left*0.5,left).premult_alpha()*left/supersample;
 			return pool.demult_alpha();
 		}
 	}
@@ -361,7 +388,6 @@ CurveGradient::color_func(const Point &point_, int quality, float supersample)co
 float
 CurveGradient::calc_supersample(const synfig::Point &x, float pw,float ph)const
 {
-//	return pw/(p2-p1).mag();
 	return pw;
 }
 
@@ -380,24 +406,11 @@ CurveGradient::hit_check(synfig::Context context, const synfig::Point &point)con
 bool
 CurveGradient::set_param(const String & param, const ValueBase &value)
 {
-	if(param=="p1" && value.same_as(p1))
-	{
-		p1=value.get(p1);
-		sync();
-		return true;
-	}
-	if(param=="p2" && value.same_as(p2))
-	{
-		p2=value.get(p2);
-		sync();
-		return true;
-	}
-	//IMPORT(p1);
-	//IMPORT(p2);
-	
-	
+
+
 	IMPORT(offset);
 	IMPORT(perpendicular);
+	IMPORT(fast);
 
 	if(param=="bline" && value.get_type()==ValueBase::TYPE_LIST)
 	{
@@ -412,7 +425,7 @@ CurveGradient::set_param(const String & param, const ValueBase &value)
 	IMPORT(gradient);
 	IMPORT(loop);
 	IMPORT(zigzag);
-	return Layer_Composite::set_param(param,value);	
+	return Layer_Composite::set_param(param,value);
 }
 
 ValueBase
@@ -420,18 +433,17 @@ CurveGradient::get_param(const String & param)const
 {
 	EXPORT(offset);
 	EXPORT(bline);
-	EXPORT(p1);
-	EXPORT(p2);
 	EXPORT(gradient);
 	EXPORT(loop);
 	EXPORT(zigzag);
 	EXPORT(width);
 	EXPORT(perpendicular);
-	
+	EXPORT(fast);
+
 	EXPORT_NAME();
 	EXPORT_VERSION();
-		
-	return Layer_Composite::get_param(param);	
+
+	return Layer_Composite::get_param(param);
 }
 
 Layer::Vocab
@@ -439,36 +451,30 @@ CurveGradient::get_param_vocab()const
 {
 	Layer::Vocab ret(Layer_Composite::get_param_vocab());
 
-		ret.push_back(ParamDesc("offset")
-		.set_local_name(_("Offset"))
-	);
+	ret.push_back(ParamDesc("offset")
+				  .set_local_name(_("Offset")));
 
 	ret.push_back(ParamDesc("width")
-		.set_is_distance()
-		.set_local_name(_("Width"))
-	);
+				  .set_is_distance()
+				  .set_local_name(_("Width")));
 
 	ret.push_back(ParamDesc("bline")
-		.set_local_name(_("Vertices"))
-		.set_origin("offset")
-		.set_scalar("width")
-		.set_description(_("A list of BLine Points"))
-	);
-
+				  .set_local_name(_("Vertices"))
+				  .set_origin("offset")
+				  .set_scalar("width")
+				  .set_description(_("A list of BLine Points")));
 
 	ret.push_back(ParamDesc("gradient")
-		.set_local_name(_("Gradient"))
-	);
+				  .set_local_name(_("Gradient")));
 	ret.push_back(ParamDesc("loop")
-		.set_local_name(_("Loop"))
-	);
+				  .set_local_name(_("Loop")));
 	ret.push_back(ParamDesc("zigzag")
-		.set_local_name(_("ZigZag"))
-	);
+				  .set_local_name(_("ZigZag")));
 	ret.push_back(ParamDesc("perpendicular")
-		.set_local_name(_("Perpendicular"))
-	);
-	
+				  .set_local_name(_("Perpendicular")));
+	ret.push_back(ParamDesc("fast")
+				  .set_local_name(_("Fast")));
+
 	return ret;
 }
 
@@ -500,7 +506,7 @@ CurveGradient::accelerated_render(Context context,Surface *surface,int quality,
 			return true;
 	}
 
-		
+
 	int x,y;
 
 	Surface::pen pen(surface->begin());
@@ -509,7 +515,7 @@ CurveGradient::accelerated_render(Context context,Surface *surface,int quality,
 	Point tl(renddesc.get_tl());
 	const int w(surface->get_w());
 	const int h(surface->get_h());
-	
+
 	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
 	{
 		for(y=0,pos[1]=tl[1];y<h;y++,pen.inc_y(),pen.dec_x(x),pos[1]+=ph)
