@@ -1,8 +1,10 @@
-/*! ========================================================================
-** Synfig
-** Template Header File
-** $Id: curvegradient.h,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file curvegradient.h
+**	\brief Template Header
 **
+**	$Id: curvegradient.h 480 2007-04-20 07:55:50Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
 **	This package is free software; you can redistribute it and/or
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -48,17 +51,16 @@ class CurveGradient : public Layer_Composite, public Layer_NoDeform
 private:
 	std::vector<synfig::BLinePoint> bline;
 
-	Point p1,p2;
 	Point offset;
 	Real width;
-	Vector diff;
 	Gradient gradient;
-	Real	curve_length_;
+	Real curve_length_;
 
 	bool loop;
 	bool zigzag;
 	bool bline_loop;
 	bool perpendicular;
+	bool fast;
 
 	void sync();
 
@@ -68,12 +70,12 @@ private:
 
 public:
 	CurveGradient();
-	
+
 	virtual bool set_param(const String &param, const ValueBase &value);
 	virtual ValueBase get_param(const String &param)const;
 	virtual Color get_color(Context context, const Point &pos)const;
 	virtual bool accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const;
-	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;	
+	synfig::Layer::Handle hit_check(synfig::Context context, const synfig::Point &point)const;
 
 	virtual Vocab get_param_vocab()const;
 };
