@@ -1,8 +1,10 @@
-/*! ========================================================================
-** Synfig
-** Template File
-** $Id: lineargradient.cpp,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file lineargradient.cpp
+**	\brief Template Header
 **
+**	$Id: lineargradient.cpp 373 2007-03-23 16:02:09Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
 **	This package is free software; you can redistribute it and/or
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -50,7 +53,7 @@ SYNFIG_LAYER_SET_NAME(LinearGradient,"linear_gradient");
 SYNFIG_LAYER_SET_LOCAL_NAME(LinearGradient,_("Linear Gradient"));
 SYNFIG_LAYER_SET_CATEGORY(LinearGradient,_("Gradients"));
 SYNFIG_LAYER_SET_VERSION(LinearGradient,"0.0");
-SYNFIG_LAYER_SET_CVS_ID(LinearGradient,"$Id: lineargradient.cpp,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $");
+SYNFIG_LAYER_SET_CVS_ID(LinearGradient,"$Id: lineargradient.cpp 373 2007-03-23 16:02:09Z dooglus $");
 
 /* === P R O C E D U R E S ================================================= */
 
@@ -79,10 +82,10 @@ inline Color
 LinearGradient::color_func(const Point &point, float supersample)const
 {
 	Real dist(point*diff-p1*diff);
-	
+
 	if(loop)
 		dist-=floor(dist);
-	
+
 	if(zigzag)
 	{
 		dist*=2.0;
@@ -94,14 +97,20 @@ LinearGradient::color_func(const Point &point, float supersample)const
 	{
 		if(dist+supersample*0.5>1.0)
 		{
-			Color pool(gradient(dist,supersample*0.5).premult_alpha()*(1.0-(dist-supersample*0.5)));
-			pool+=gradient((dist+supersample*0.5)-1.0,supersample*0.5).premult_alpha()*((dist+supersample*0.5)-1.0);
+			float  left(supersample*0.5-(dist-1.0));
+			float right(supersample*0.5+(dist-1.0));
+			Color pool(gradient(1.0-(left*0.5),left).premult_alpha()*left/supersample);
+			if (zigzag) pool+=gradient(1.0-right*0.5,right).premult_alpha()*right/supersample;
+			else		pool+=gradient(right*0.5,right).premult_alpha()*right/supersample;
 			return pool.demult_alpha();
 		}
 		if(dist-supersample*0.5<0.0)
 		{
-			Color pool(gradient(dist,supersample*0.5).premult_alpha()*(dist+supersample*0.5));
-			pool+=gradient(1.0-(dist-supersample*0.5),supersample*0.5).premult_alpha()*(-(dist-supersample*0.5));
+			float  left(supersample*0.5-dist);
+			float right(supersample*0.5+dist);
+			Color pool(gradient(right*0.5,right).premult_alpha()*right/supersample);
+			if (zigzag) pool+=gradient(left*0.5,left).premult_alpha()*left/supersample;
+			else		pool+=gradient(1.0-left*0.5,left).premult_alpha()*left/supersample;
 			return pool.demult_alpha();
 		}
 	}
@@ -143,12 +152,12 @@ LinearGradient::set_param(const String & param, const ValueBase &value)
 	}
 	//IMPORT(p1);
 	//IMPORT(p2);
-	
-	
+
+
 	IMPORT(gradient);
 	IMPORT(loop);
 	IMPORT(zigzag);
-	return Layer_Composite::set_param(param,value);	
+	return Layer_Composite::set_param(param,value);
 }
 
 ValueBase
@@ -159,18 +168,18 @@ LinearGradient::get_param(const String & param)const
 	EXPORT(gradient);
 	EXPORT(loop);
 	EXPORT(zigzag);
-	
+
 	EXPORT_NAME();
 	EXPORT_VERSION();
-		
-	return Layer_Composite::get_param(param);	
+
+	return Layer_Composite::get_param(param);
 }
 
 Layer::Vocab
 LinearGradient::get_param_vocab()const
 {
 	Layer::Vocab ret(Layer_Composite::get_param_vocab());
-	
+
 	ret.push_back(ParamDesc("p1")
 		.set_local_name(_("Point 1"))
 		.set_connect("p2")
@@ -187,7 +196,7 @@ LinearGradient::get_param_vocab()const
 	ret.push_back(ParamDesc("zigzag")
 		.set_local_name(_("ZigZag"))
 	);
-	
+
 	return ret;
 }
 
@@ -219,7 +228,7 @@ LinearGradient::accelerated_render(Context context,Surface *surface,int quality,
 			return true;
 	}
 
-		
+
 	int x,y;
 
 	Surface::pen pen(surface->begin());
@@ -228,7 +237,7 @@ LinearGradient::accelerated_render(Context context,Surface *surface,int quality,
 	Point tl(renddesc.get_tl());
 	const int w(surface->get_w());
 	const int h(surface->get_h());
-	
+
 	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
 	{
 		for(y=0,pos[1]=tl[1];y<h;y++,pen.inc_y(),pen.dec_x(x),pos[1]+=ph)
