@@ -1,8 +1,8 @@
 /* === S Y N F I G ========================================================= */
-/*!	\file layer_solidcolor.cpp
+/*!	\file mod_gradient/radialgradient.cpp
 **	\brief Template Header
 **
-**	$Id: radialgradient.cpp,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $
+**	$Id: radialgradient.cpp 366 2007-03-22 18:28:52Z dooglus $
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
@@ -55,7 +55,7 @@ SYNFIG_LAYER_SET_NAME(RadialGradient,"radial_gradient");
 SYNFIG_LAYER_SET_LOCAL_NAME(RadialGradient,_("Radial Gradient"));
 SYNFIG_LAYER_SET_CATEGORY(RadialGradient,_("Gradients"));
 SYNFIG_LAYER_SET_VERSION(RadialGradient,"0.1");
-SYNFIG_LAYER_SET_CVS_ID(RadialGradient,"$Id: radialgradient.cpp,v 1.1.1.1 2005/01/04 01:23:10 darco Exp $");
+SYNFIG_LAYER_SET_CVS_ID(RadialGradient,"$Id: radialgradient.cpp 366 2007-03-22 18:28:52Z dooglus $");
 
 /* === P R O C E D U R E S ================================================= */
 
@@ -72,7 +72,7 @@ RadialGradient::RadialGradient():
 	zigzag(false)
 {
 }
-	
+
 bool
 RadialGradient::set_param(const String & param, const ValueBase &value)
 {
@@ -81,7 +81,7 @@ RadialGradient::set_param(const String & param, const ValueBase &value)
 	IMPORT(radius);
 	IMPORT(loop);
 	IMPORT(zigzag);
-	
+
 	return Layer_Composite::set_param(param,value);
 }
 
@@ -93,18 +93,18 @@ RadialGradient::get_param(const String &param)const
 	EXPORT(radius);
 	EXPORT(loop);
 	EXPORT(zigzag);
-	
+
 	EXPORT_NAME();
 	EXPORT_VERSION();
-		
-	return Layer_Composite::get_param(param);	
+
+	return Layer_Composite::get_param(param);
 }
 
 Layer::Vocab
 RadialGradient::get_param_vocab()const
 {
 	Layer::Vocab ret(Layer_Composite::get_param_vocab());
-	
+
 	ret.push_back(ParamDesc("gradient")
 		.set_local_name(_("Gradient"))
 	);
@@ -112,7 +112,7 @@ RadialGradient::get_param_vocab()const
 	ret.push_back(ParamDesc("center")
 		.set_local_name(_("Center"))
 	);
-	
+
 	ret.push_back(ParamDesc("radius")
 		.set_local_name(_("Radius"))
 		.set_description(_("This is the radius of the circle"))
@@ -127,7 +127,7 @@ RadialGradient::get_param_vocab()const
 	ret.push_back(ParamDesc("zigzag")
 		.set_local_name(_("Zig-Zag"))
 	);
-	
+
 	return ret;
 }
 
@@ -135,7 +135,7 @@ inline Color
 RadialGradient::color_func(const Point &point, float supersample)const
 {
 	Real dist((point-center).mag()/radius);
-	
+
 	if(zigzag)
 	{
 		dist*=2.0;
@@ -149,18 +149,24 @@ RadialGradient::color_func(const Point &point, float supersample)const
 
 		if(dist+supersample*0.5>1.0)
 		{
-			Color pool(gradient(dist,supersample*0.5).premult_alpha()*(1.0-(dist-supersample*0.5)));
-			pool+=gradient((dist+supersample*0.5)-1.0,supersample*0.5).premult_alpha()*((dist+supersample*0.5)-1.0);
+			float  left(supersample*0.5-(dist-1.0));
+			float right(supersample*0.5+(dist-1.0));
+			Color pool(gradient(1.0-(left*0.5),left).premult_alpha()*left/supersample);
+			if (zigzag) pool+=gradient(1.0-right*0.5,right).premult_alpha()*right/supersample;
+			else		pool+=gradient(right*0.5,right).premult_alpha()*right/supersample;
 			return pool.demult_alpha();
 		}
 		if(dist-supersample*0.5<0.0)
 		{
-			Color pool(gradient(dist,supersample*0.5).premult_alpha()*(dist+supersample*0.5));
-			pool+=gradient(1.0-(dist-supersample*0.5),supersample*0.5).premult_alpha()*(-(dist-supersample*0.5));
+			float  left(supersample*0.5-dist);
+			float right(supersample*0.5+dist);
+			Color pool(gradient(right*0.5,right).premult_alpha()*right/supersample);
+			if (zigzag) pool+=gradient(left*0.5,left).premult_alpha()*left/supersample;
+			else		pool+=gradient(1.0-left*0.5,left).premult_alpha()*left/supersample;
 			return pool.demult_alpha();
 		}
 	}
-	
+
 	return gradient(dist,supersample);
 }
 
@@ -194,7 +200,7 @@ RadialGradient::get_color(Context context, const Point &pos)const
 	else
 		return Color::blend(color,context.get_color(pos),get_amount(),get_blend_method());
 }
-	
+
 bool
 RadialGradient::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
@@ -212,7 +218,7 @@ RadialGradient::accelerated_render(Context context,Surface *surface,int quality,
 			return true;
 	}
 
-		
+
 	int x,y;
 
 	Surface::pen pen(surface->begin());
@@ -221,7 +227,7 @@ RadialGradient::accelerated_render(Context context,Surface *surface,int quality,
 	Point tl(renddesc.get_tl());
 	const int w(surface->get_w());
 	const int h(surface->get_h());
-	
+
 	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
 	{
 		for(y=0,pos[1]=tl[1];y<h;y++,pen.inc_y(),pen.dec_x(x),pos[1]+=ph)
