@@ -2,7 +2,7 @@
 /*!	\file spiralgradient.cpp
 **	\brief Template Header
 **
-**	$Id: spiralgradient.cpp,v 1.1.1.1 2005/01/04 01:23:11 darco Exp $
+**	$Id: spiralgradient.cpp 367 2007-03-22 18:46:09Z dooglus $
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
@@ -55,7 +55,7 @@ SYNFIG_LAYER_SET_NAME(SpiralGradient,"spiral_gradient");
 SYNFIG_LAYER_SET_LOCAL_NAME(SpiralGradient,_("Spiral Gradient"));
 SYNFIG_LAYER_SET_CATEGORY(SpiralGradient,_("Gradients"));
 SYNFIG_LAYER_SET_VERSION(SpiralGradient,"0.1");
-SYNFIG_LAYER_SET_CVS_ID(SpiralGradient,"$Id: spiralgradient.cpp,v 1.1.1.1 2005/01/04 01:23:11 darco Exp $");
+SYNFIG_LAYER_SET_CVS_ID(SpiralGradient,"$Id: spiralgradient.cpp 367 2007-03-22 18:46:09Z dooglus $");
 
 /* === P R O C E D U R E S ================================================= */
 
@@ -72,7 +72,7 @@ SpiralGradient::SpiralGradient():
 	clockwise(false)
 {
 }
-	
+
 bool
 SpiralGradient::set_param(const String & param, const ValueBase &value)
 {
@@ -81,7 +81,7 @@ SpiralGradient::set_param(const String & param, const ValueBase &value)
 	IMPORT(radius);
 	IMPORT(angle);
 	IMPORT(clockwise);
-	
+
 	return Layer_Composite::set_param(param,value);
 }
 
@@ -93,18 +93,18 @@ SpiralGradient::get_param(const String &param)const
 	EXPORT(radius);
 	EXPORT(angle);
 	EXPORT(clockwise);
-	
+
 	EXPORT_NAME();
 	EXPORT_VERSION();
-		
-	return Layer_Composite::get_param(param);	
+
+	return Layer_Composite::get_param(param);
 }
 
 Layer::Vocab
 SpiralGradient::get_param_vocab()const
 {
 	Layer::Vocab ret(Layer_Composite::get_param_vocab());
-	
+
 	ret.push_back(ParamDesc("gradient")
 		.set_local_name(_("Gradient"))
 	);
@@ -112,7 +112,7 @@ SpiralGradient::get_param_vocab()const
 	ret.push_back(ParamDesc("center")
 		.set_local_name(_("Center"))
 	);
-	
+
 	ret.push_back(ParamDesc("radius")
 		.set_local_name(_("Radius"))
 		.set_description(_("This is the radius of the circle"))
@@ -128,7 +128,7 @@ SpiralGradient::get_param_vocab()const
 	ret.push_back(ParamDesc("clockwise")
 		.set_local_name(_("Clockwise"))
 	);
-	
+
 	return ret;
 }
 
@@ -139,29 +139,33 @@ SpiralGradient::color_func(const Point &pos, float supersample)const
 	Angle a;
 	a=Angle::tan(-centered[1],centered[0]).mod();
 	a=a+angle;
-	
+
 	if(supersample<0.00001)supersample=0.00001;
-	
+
 	Real dist((pos-center).mag()/radius);
 	if(clockwise)
 		dist+=Angle::rot(a.mod()).get();
 	else
 		dist-=Angle::rot(a.mod()).get();
-		
+
 	dist-=floor(dist);
 	if(dist+supersample*0.5>1.0)
 	{
-		Color pool(gradient(dist,supersample*0.5).premult_alpha()*(1.0-(dist-supersample*0.5)));
-		pool+=gradient((dist+supersample*0.5)-1.0,supersample*0.5).premult_alpha()*((dist+supersample*0.5)-1.0);
+		float  left(supersample*0.5-(dist-1.0));
+		float right(supersample*0.5+(dist-1.0));
+		Color pool(gradient(1.0-(left*0.5),left).premult_alpha()*left/supersample);
+		pool+=gradient(right*0.5,right).premult_alpha()*right/supersample;
 		return pool.demult_alpha();
 	}
 	if(dist-supersample*0.5<0.0)
 	{
-		Color pool(gradient(dist,supersample*0.5).premult_alpha()*(dist+supersample*0.5));
-		pool+=gradient(1.0-(dist-supersample*0.5),supersample*0.5).premult_alpha()*(-(dist-supersample*0.5));
+		float  left(supersample*0.5-dist);
+		float right(supersample*0.5+dist);
+		Color pool(gradient(right*0.5,right).premult_alpha()*right/supersample);
+		pool+=gradient(1.0-left*0.5,left).premult_alpha()*left/supersample;
 		return pool.demult_alpha();
 	}
-	
+
 	return gradient(dist,supersample);
 }
 
@@ -187,13 +191,13 @@ Color
 SpiralGradient::get_color(Context context, const Point &pos)const
 {
 	const Color color(color_func(pos));
-	
+
 	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
 		return color;
 	else
 		return Color::blend(color,context.get_color(pos),get_amount(),get_blend_method());
 }
-	
+
 bool
 SpiralGradient::accelerated_render(Context context,Surface *surface,int quality, const RendDesc &renddesc, ProgressCallback *cb)const
 {
@@ -211,7 +215,7 @@ SpiralGradient::accelerated_render(Context context,Surface *surface,int quality,
 			return true;
 	}
 
-		
+
 	int x,y;
 
 	Surface::pen pen(surface->begin());
@@ -220,7 +224,7 @@ SpiralGradient::accelerated_render(Context context,Surface *surface,int quality,
 	Point tl(renddesc.get_tl());
 	const int w(surface->get_w());
 	const int h(surface->get_h());
-	
+
 	if(get_amount()==1.0 && get_blend_method()==Color::BLEND_STRAIGHT)
 	{
 		for(y=0,pos[1]=tl[1];y<h;y++,pen.inc_y(),pen.dec_x(x),pos[1]+=ph)
