@@ -1,7 +1,9 @@
-/*! ========================================================================
-** Synfig
-** ppm Target Module
-** $Id: trgt_imagemagick.cpp,v 1.1.1.1 2005/01/04 01:23:11 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file trgt_imagemagick.cpp
+**	\brief ppm Target Module
+**
+**	\legal
+** $Id: trgt_imagemagick.cpp 335 2007-03-16 00:39:09Z dooglus $
 **
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -52,7 +55,7 @@ SYNFIG_TARGET_INIT(imagemagick_trgt);
 SYNFIG_TARGET_SET_NAME(imagemagick_trgt,"imagemagick");
 SYNFIG_TARGET_SET_EXT(imagemagick_trgt,"miff");
 SYNFIG_TARGET_SET_VERSION(imagemagick_trgt,"0.1");
-SYNFIG_TARGET_SET_CVS_ID(imagemagick_trgt,"$Id: trgt_imagemagick.cpp,v 1.1.1.1 2005/01/04 01:23:11 darco Exp $");
+SYNFIG_TARGET_SET_CVS_ID(imagemagick_trgt,"$Id: trgt_imagemagick.cpp 335 2007-03-16 00:39:09Z dooglus $");
 
 /* === M E T H O D S ======================================================= */
 
@@ -133,7 +136,7 @@ imagemagick_trgt::start_frame(synfig::ProgressCallback *cb)
 		return false;
 	}
 
-	//etl::yield();	
+	//etl::yield();
 
 	return true;
 }
@@ -154,6 +157,6 @@ imagemagick_trgt::end_scanline(void)
 
 	if(!fwrite(buffer,channels(pf),desc.get_w(),file))
 		return false;
-	
+
 	return true;
 }
