@@ -1,8 +1,10 @@
-/*! ========================================================================
-** Synfig
-** jpeg_trgt Target Module
-** $Id: trgt_jpeg.cpp,v 1.1.1.1 2005/01/04 01:23:11 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file trgt_jpeg.cpp
+**	\brief jpeg_trgt Target Module
 **
+**	$Id: trgt_jpeg.cpp 373 2007-03-23 16:02:09Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
 **	This package is free software; you can redistribute it and/or
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -50,7 +53,7 @@ SYNFIG_TARGET_INIT(jpeg_trgt);
 SYNFIG_TARGET_SET_NAME(jpeg_trgt,"jpeg");
 SYNFIG_TARGET_SET_EXT(jpeg_trgt,"jpg");
 SYNFIG_TARGET_SET_VERSION(jpeg_trgt,"0.1");
-SYNFIG_TARGET_SET_CVS_ID(jpeg_trgt,"$Id: trgt_jpeg.cpp,v 1.1.1.1 2005/01/04 01:23:11 darco Exp $");
+SYNFIG_TARGET_SET_CVS_ID(jpeg_trgt,"$Id: trgt_jpeg.cpp 373 2007-03-23 16:02:09Z dooglus $");
 
 /* === M E T H O D S ======================================================= */
 
@@ -61,7 +64,7 @@ jpeg_trgt::jpeg_trgt(const char *Filename)
 	buffer=NULL;
 	ready=false;
 	quality=95;
-	color_buffer=0;	
+	color_buffer=0;
 	set_remove_alpha();
 }
 
@@ -96,7 +99,7 @@ bool
 jpeg_trgt::start_frame(synfig::ProgressCallback *callback)
 {
 	int w=desc.get_w(),h=desc.get_h();
-	
+
 	if(file && file!=stdout)
 		fclose(file);
 	if(filename=="-")
@@ -110,7 +113,7 @@ jpeg_trgt::start_frame(synfig::ProgressCallback *callback)
 			newfilename(filename),
 			ext(find(filename.begin(),filename.end(),'.'),filename.end());
 		newfilename.erase(find(newfilename.begin(),newfilename.end(),'.'),newfilename.end());
-		
+
 		newfilename+=etl::strprintf("%04d",imagecount)+ext;
 		file=fopen(newfilename.c_str(),"wb");
 		if(callback)callback->task(newfilename);
@@ -120,13 +123,13 @@ jpeg_trgt::start_frame(synfig::ProgressCallback *callback)
 		file=fopen(filename.c_str(),"wb");
 		if(callback)callback->task(filename);
 	}
-	
+
 	if(!file)
 		return false;
-		
+
 	delete [] buffer;
 	buffer=new unsigned char[3*w];
-	
+
 	delete [] color_buffer;
 	color_buffer=new Color[w];
 
@@ -148,14 +151,14 @@ jpeg_trgt::start_frame(synfig::ProgressCallback *callback)
 	* Here we just illustrate the use of quality (quantization table) scaling:
 	*/
 	jpeg_set_quality(&cinfo, quality, TRUE /* limit to baseline-JPEG values */);
-	
+
 	/* Step 4: Start compressor */
-	
+
 	/* TRUE ensures that we will write a complete interchange-JPEG file.
 	* Pass TRUE unless you are very sure of what you're doing.
 	*/
 	jpeg_start_compress(&cinfo, TRUE);
-		
+
 	ready=true;
 	return true;
 }
