@@ -1,8 +1,10 @@
-/*! ========================================================================
-** Synfig
-** ppm Target Module
-** $Id: mptr_openexr.cpp,v 1.1.1.1 2005/01/04 01:23:14 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file mptr_openexr.cpp
+**	\brief ppm Target Module
 **
+**	$Id: mptr_openexr.cpp 476 2007-04-18 09:55:49Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
 **	This package is free software; you can redistribute it and/or
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -29,7 +32,8 @@
 #endif
 
 #include "mptr_openexr.h"
-#include <synfig/synfig.h>
+#include <synfig/general.h>
+#include <synfig/surface.h>
 #include <ETL/stringf>
 #include <cstdio>
 #include <algorithm>
@@ -50,10 +54,10 @@ using namespace etl;
 /* === G L O B A L S ======================================================= */
 
 SYNFIG_IMPORTER_INIT(exr_mptr);
-SYNFIG_IMPORTER_SET_NAME(exr_mptr,"exr_mptr");
+SYNFIG_IMPORTER_SET_NAME(exr_mptr,"openexr");
 SYNFIG_IMPORTER_SET_EXT(exr_mptr,"exr");
 SYNFIG_IMPORTER_SET_VERSION(exr_mptr,"0.1");
-SYNFIG_IMPORTER_SET_CVS_ID(exr_mptr,"$Id: mptr_openexr.cpp,v 1.1.1.1 2005/01/04 01:23:14 darco Exp $");
+SYNFIG_IMPORTER_SET_CVS_ID(exr_mptr,"$Id: mptr_openexr.cpp 476 2007-04-18 09:55:49Z dooglus $");
 
 /* === M E T H O D S ======================================================= */
 
@@ -91,7 +95,7 @@ exr_mptr::get_frame(synfig::Surface &out_surface,Time, synfig::ProgressCallback
 	in.setFrameBuffer (reinterpret_cast<Imf::Rgba *>(in_surface[0]), 1, w);
 
 	in.readPixels (in.dataWindow().min.y, in.dataWindow().max.y);
-	
+
 	int x;
 	int y;
 	out_surface.set_wh(w,h);
@@ -114,5 +118,5 @@ exr_mptr::get_frame(synfig::Surface &out_surface,Time, synfig::ProgressCallback
 		return false;
     }
 
-	return true;		
+	return true;
 }
