@@ -1,8 +1,10 @@
-/*! ========================================================================
-** Synfig
-** png_trgt Target Module
-** $Id: trgt_png.cpp,v 1.1.1.1 2005/01/04 01:23:14 darco Exp $
+/* === S Y N F I G ========================================================= */
+/*!	\file trgt_png.cpp
+**	\brief png_trgt Target Module
 **
+**	$Id: trgt_png.cpp 474 2007-04-18 09:45:40Z dooglus $
+**
+**	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **
 **	This package is free software; you can redistribute it and/or
@@ -14,6 +16,7 @@
 **	but WITHOUT ANY WARRANTY; without even the implied warranty of
 **	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **	General Public License for more details.
+**	\endlegal
 **
 ** === N O T E S ===========================================================
 **
@@ -52,7 +55,7 @@ SYNFIG_TARGET_INIT(png_trgt);
 SYNFIG_TARGET_SET_NAME(png_trgt,"png");
 SYNFIG_TARGET_SET_EXT(png_trgt,"png");
 SYNFIG_TARGET_SET_VERSION(png_trgt,"0.1");
-SYNFIG_TARGET_SET_CVS_ID(png_trgt,"$Id: trgt_png.cpp,v 1.1.1.1 2005/01/04 01:23:14 darco Exp $");
+SYNFIG_TARGET_SET_CVS_ID(png_trgt,"$Id: trgt_png.cpp 474 2007-04-18 09:45:40Z dooglus $");
 
 /* === M E T H O D S ======================================================= */
 
@@ -81,7 +84,7 @@ png_trgt::png_trgt(const char *Filename)
 	filename=Filename;
 	buffer=NULL;
 	ready=false;
-	color_buffer=0;	
+	color_buffer=0;
 }
 
 png_trgt::~png_trgt()
@@ -126,7 +129,7 @@ bool
 png_trgt::start_frame(synfig::ProgressCallback *callback)
 {
 	int w=desc.get_w(),h=desc.get_h();
-	
+
 	if(file && file!=stdout)
 		fclose(file);
 	if(filename=="-")
@@ -140,7 +143,7 @@ png_trgt::start_frame(synfig::ProgressCallback *callback)
 			newfilename(filename),
 			ext(find(filename.begin(),filename.end(),'.'),filename.end());
 		newfilename.erase(find(newfilename.begin(),newfilename.end(),'.'),newfilename.end());
-		
+
 		newfilename+=etl::strprintf("%04d",imagecount)+ext;
 		file=fopen(newfilename.c_str(),"wb");
 		if(callback)callback->task(newfilename);
@@ -150,10 +153,10 @@ png_trgt::start_frame(synfig::ProgressCallback *callback)
 		file=fopen(filename.c_str(),"wb");
 		if(callback)callback->task(filename);
 	}
-	
+
 	if(!file)
 		return false;
-		
+
 	delete [] buffer;
 	buffer=new unsigned char[4*w];
 
@@ -167,7 +170,7 @@ png_trgt::start_frame(synfig::ProgressCallback *callback)
 		fclose(file);
 		return false;
 	}
-	
+
 	info_ptr= png_create_info_struct(png_ptr);
 	if (!info_ptr)
 	{
@@ -176,7 +179,7 @@ png_trgt::start_frame(synfig::ProgressCallback *callback)
 		png_destroy_write_struct(&png_ptr,(png_infopp)NULL);
 		return false;
 	}
-	
+
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		synfig::error("Unable to setup longjump");
@@ -193,20 +196,24 @@ png_trgt::start_frame(synfig::ProgressCallback *callback)
 	// Write the gamma
 	//png_set_gAMA(png_ptr, info_ptr,1.0/gamma().get_gamma());
 	png_set_gAMA(png_ptr, info_ptr,gamma().get_gamma());
-	
+
 	// Write the physical size
 	png_set_pHYs(png_ptr,info_ptr,round_to_int(desc.get_x_res()),round_to_int(desc.get_y_res()),PNG_RESOLUTION_METER);
-	
+
 	// Output any text info along with the file
 	png_text comments[]=
 	{
-		{ PNG_TEXT_COMPRESSION_NONE, "Title", const_cast<char *>(get_canvas()->get_name().c_str()) },
-		{ PNG_TEXT_COMPRESSION_NONE, "Description", const_cast<char *>(get_canvas()->get_description().c_str()) },
-//		{ PNG_TEXT_COMPRESSION_NONE, "Copyright", "(c) 2004 Voria Studios, LLC" },
-		{ PNG_TEXT_COMPRESSION_NONE, "Software", "SYNFIG" },
+		{ PNG_TEXT_COMPRESSION_NONE, "Title", const_cast<char *>(get_canvas()->get_name().c_str()),
+		  strlen(get_canvas()->get_name().c_str()) },
+		{ PNG_TEXT_COMPRESSION_NONE, "Description", const_cast<char *>(get_canvas()->get_description().c_str()),
+		  strlen(get_canvas()->get_description().c_str()) },
+//		{ PNG_TEXT_COMPRESSION_NONE, "Copyright", "(c) 2004 Voria Studios, LLC",
+//		  strlen("(c) 2004 Voria Studios, LLC") },
+		{ PNG_TEXT_COMPRESSION_NONE, "Software", "SYNFIG",
+		  strlen("SYNFIG") },
 	};
 	png_set_text(png_ptr,info_ptr,comments,sizeof(comments)/sizeof(png_text));
-	
+
 	png_write_info_before_PLTE(png_ptr, info_ptr);
 	png_write_info(png_ptr, info_ptr);
 	ready=true;
