@@ -2,7 +2,7 @@
 /*!	\file layer_composite.h
 **	\brief Composite Layer Class Implementation
 **
-**	$Id: layer_composite.h,v 1.2 2005/01/24 03:08:18 darco Exp $
+**	$Id: layer_composite.h 401 2007-03-31 22:11:13Z dooglus $
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
@@ -72,10 +72,10 @@ public:
 
 	Layer_Composite& set_blend_method(Color::BlendMethod x) { blend_method_=x; return *this; }
 
-	bool is_solid_color()const { return amount_==1.0f && blend_method_==Color::BLEND_STRAIGHT; }
-	
+	virtual bool is_solid_color()const { return amount_==1.0f && blend_method_==Color::BLEND_STRAIGHT; }
+
 	bool is_disabled()const { return amount_==0.0f; }
-	
+
 	virtual Vocab get_param_vocab()const;
 
 	virtual bool set_param(const String &param, const ValueBase &value);
