@@ -2,7 +2,7 @@
 /*!	\file time.cpp
 **	\brief Template File
 **
-**	$Id: time.cpp,v 1.1.1.1 2005/01/04 01:23:15 darco Exp $
+**	$Id: time.cpp 482 2007-05-06 22:19:03Z dooglus $
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
@@ -35,10 +35,10 @@
 #include "general.h"
 #include <cmath>
 #include <cassert>
+#include <algorithm>
 #include <ctype.h>
 #include <math.h>
 
-#ifndef isnan
 
 #ifdef WIN32
 #include <float.h>
@@ -48,11 +48,17 @@ extern "C" { int _isnan(double x); }
 #endif
 #endif
 
-#ifdef __APPLE__
-#define isnan __isnanf
+// For some reason isnan() isn't working on macosx any more.
+// This is a quick fix.
+#if defined(__APPLE__) && !defined(SYNFIG_ISNAN_FIX)
+#ifdef isnan
+#undef isnan
+#endif
+inline bool isnan(double x) { return x != x; }
+inline bool isnan(float x) { return x != x; }
+#define SYNFIG_ISNAN_FIX 1
 #endif
 
-#endif
 
 #endif
 
@@ -77,13 +83,13 @@ Time::Time(const String &str_, float fps):
 	std::transform(str.begin(),str.end(),str.begin(),&tolower);
 
 	// Start/Begin Of Time
-	if(str=="SOT" || str=="BOT")
+	if(str=="sot" || str=="bot")
 	{
 		operator=(begin());
 		return;
 	}
 	// End Of Time
-	if(str=="EOT")
+	if(str=="eot")
 	{
 		operator=(end());
 		return;
@@ -93,7 +99,7 @@ Time::Time(const String &str_, float fps):
 	unsigned int pos=0;
 	int read;
 	float amount;
-	
+
 	// Now try to read it in the letter-abreviated format
 	while(pos<str.size() && sscanf(String(str,pos).c_str(),"%f%n",&amount,&read))
 	{
@@ -140,7 +146,7 @@ Time::Time(const String &str_, float fps):
 							value_=frame/fps+(hour*3600+minute*60+second);
 							return;
 					}
-			
+
 					if(sscanf(str.c_str(),"%d:%d:%d",&hour,&minute,&second)==3)
 					{
 						value_=hour*3600+minute*60+second;
@@ -164,22 +170,23 @@ String
 Time::get_string(float fps, Time::Format format)const
 {
 	Time time(*this);
-	
+
 	if(time<=begin())
 		return "SOT";	// Start Of Time
 	if(time>=end())
 		return "EOT";	// End Of Time
-	
+
 	if(fps<0)fps=0;
-	
+
 	if(ceil(time.value_)-time.value_<epsilon_())
 		time.value_=ceil(time.value_);
-	
+
 	int hour,minute;
-	
+
 	hour=time/3600;time-=hour*3600;
 	minute=time/60;time-=minute*60;
-	
+
+	// <= is redefined, so this means "is the FORMAT_VIDEO bit set in the format?"
 	if(format<=FORMAT_VIDEO)
 	{
 		int second;
@@ -195,15 +202,22 @@ Time::get_string(float fps, Time::Format format)const
 		else
 			return strprintf("%02d:%02d:%02d",hour,minute,second);
 	}
-	
+
 	String ret;
+	bool started = false;
 
 	if(format<=FORMAT_FULL || hour)
+	{
 		ret+=strprintf(format<=FORMAT_NOSPACES?"%dh":"%dh ",hour);
-	
-	if(format<=FORMAT_FULL || hour || minute)
+		started = true;
+	}
+
+	if(format<=FORMAT_FULL || minute)
+	{
 		ret+=strprintf(format<=FORMAT_NOSPACES?"%dm":"%dm ",minute);
-	
+		started = true;
+	}
+
 	if(fps)
 	{
 		int second;
@@ -211,23 +225,32 @@ Time::get_string(float fps, Time::Format format)const
 		second=time;time-=second;
 		frame=time*fps;
 		if(format<=FORMAT_FULL || second)
+		{
 			ret+=strprintf(format<=FORMAT_NOSPACES?"%ds":"%ds ",(int)second);
-		
-		if(abs(frame-floor(frame)>=epsilon_()))
-			ret+=strprintf("%0.3ff",frame);
-		else
-			ret+=strprintf("%0.0ff",frame);
+			started = true;
+		}
+
+		if(format<=FORMAT_FULL || frame || !started)
+		{
+			if(abs(frame-floor(frame)>=epsilon_()))
+				ret+=strprintf("%0.3ff",frame);
+			else
+				ret+=strprintf("%0.0ff",frame);
+		}
 	}
 	else
 	{
 		float second;
 		second=time;
-		if(abs(second-floor(second))>=epsilon_())
-			ret+=strprintf("%0.8fs",second);
-		else
-			ret+=strprintf("%0.0fs",second);
+		if(format<=FORMAT_FULL || second || !started)
+		{
+			if(abs(second-floor(second))>=epsilon_())
+				ret+=strprintf("%0.8fs",second);
+			else
+				ret+=strprintf("%0.0fs",second);
+		}
 	}
-	
+
 	return ret;
 }
 
