@@ -1,8 +1,8 @@
 /* === S Y N F I G ========================================================= */
-/*!	\file main.cpp
+/*!	\file tool/main.cpp
 **	\brief SYNFIG Tool
 **
-**	$Id: main.cpp,v 1.9 2005/01/23 04:41:10 darco Exp $
+**	$Id: main.cpp 444 2007-04-08 03:42:41Z dooglus $
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
@@ -62,12 +62,13 @@ enum exit_code
 	SYNFIGTOOL_FILENOTFOUND		=1,
 	SYNFIGTOOL_BORRED			=2,
 	SYNFIGTOOL_HELP				=3,
-	SYNFIGTOOL_UNKNOWNARGUMENT	=4,	
-	SYNFIGTOOL_UNKNOWNERROR		=5,	
-	SYNFIGTOOL_INVALIDTARGET		=6,	
-	SYNFIGTOOL_RENDERFAILURE		=7,	
+	SYNFIGTOOL_UNKNOWNARGUMENT	=4,
+	SYNFIGTOOL_UNKNOWNERROR		=5,
+	SYNFIGTOOL_INVALIDTARGET		=6,
+	SYNFIGTOOL_RENDERFAILURE		=7,
 	SYNFIGTOOL_BLANK				=8,
-	SYNFIGTOOL_BADVERSION		=9
+	SYNFIGTOOL_BADVERSION		=9,
+	SYNFIGTOOL_MISSINGARGUMENT	=10
 };
 
 #ifndef VERSION
@@ -94,11 +95,11 @@ bool print_benchmarks=false;
 class Progress : public synfig::ProgressCallback
 {
 	const char *program;
-	
+
 public:
-	
+
 	Progress(const char *name):program(name) { }
-	
+
 	virtual bool
 	task(const String &task)
 	{
@@ -130,16 +131,16 @@ public:
 class RenderProgress : public synfig::ProgressCallback
 {
 	string taskname;
-	
+
 	etl::clock clk;
 	int clk_scanline; // The scanline at which the clock was reset
 	etl::clock clk2;
 
-	float last_time;	
+	float last_time;
 public:
-	
-	RenderProgress():clk_scanline(0) { }
-	
+
+	RenderProgress():clk_scanline(0), last_time(0) { }
+
 	virtual bool
 	task(const String &thetask)
 	{
@@ -177,7 +178,7 @@ public:
 			if(clk2()<0.2)
 				return true;
 			clk2.reset();
-				
+
 			if(scanline)
 				seconds=(int)time+1;
 			else
@@ -186,7 +187,7 @@ public:
 				clk.reset();
 				clk_scanline=scanline;
 			}
-			
+
 			if(seconds<0)
 			{
 				clk.reset();
@@ -201,7 +202,7 @@ public:
 				days++,hours-=24;
 			while(days>=7)
 				weeks++,days-=7;
-			
+
 			cerr<<taskname<<": "<<_("Line")<<" "<<scanline<<_(" of ")<<h<<" -- ";
 			//cerr<<time/(h-clk_scanline)<<" ";
 			/*
@@ -212,9 +213,9 @@ public:
 			{
 				//cerr<<"reset"<<endl;
 				clk.reset();
-				clk_scanline=scanline;				
+				clk_scanline=scanline;
 			}
-			
+
 			if(weeks)
 				cerr<<weeks<<"w ";
 			if(days)
@@ -225,7 +226,7 @@ public:
 				cerr<<minutes<<"m ";
 			if(seconds)
 				cerr<<seconds<<"s ";
-			
+
 			cerr<<"           \r";
 		}
 		else
@@ -238,13 +239,13 @@ struct Job
 {
 	String filename;
 	String outfilename;
-	
+
 	RendDesc desc;
 
 	Canvas::Handle root;
 	Canvas::Handle canvas;
 	Target::Handle target;
-	
+
 	int quality;
 	bool sifout;
 };
@@ -294,7 +295,7 @@ void display_help(int amount)
 				cerr<<strprintf(" %s %s %s",flag, arg, spaces+strlen(arg)+strlen(flag)+1)+description<<endl;
 			else
 				cerr<<strprintf(" %s %s",flag,spaces+strlen(flag))+description<<endl;
-		
+
 		}
 	};
 	cerr<<_("syntax: ")<<progname<<" [DEFAULT OPTIONS] ([SIF FILE] [SPECIFIC OPTIONS])..."<<endl;
@@ -347,7 +348,7 @@ void display_help(int amount)
 int process_global_flags(arg_list_t &arg_list)
 {
 	arg_list_t::iterator iter, next;
-	
+
 	for(next=arg_list.begin(),iter=next++;iter!=arg_list.end();iter=next++)
 	{
 		if(*iter == "--")
@@ -358,14 +359,14 @@ int process_global_flags(arg_list_t &arg_list)
 			signal_test();
 			return SYNFIGTOOL_HELP;
 		}
-		
+
 		if(*iter == "--guid-test")
 		{
 			guid_test();
 			return SYNFIGTOOL_HELP;
 		}
 
-		
+
 		if(*iter == "--help")
 		{
 			display_help(1);
@@ -445,7 +446,7 @@ int process_global_flags(arg_list_t &arg_list)
 			synfig::Target::Book::iterator iter=synfig::Target::book().begin();
 			for(;iter!=synfig::Target::book().end();iter++)
 				cout<<iter->first<<endl;
-				
+
 			return SYNFIGTOOL_HELP;
 		}
 
@@ -456,7 +457,7 @@ int process_global_flags(arg_list_t &arg_list)
 			synfig::LinkableValueNode::Book::iterator iter=synfig::LinkableValueNode::book().begin();
 			for(;iter!=synfig::LinkableValueNode::book().end();iter++)
 				cout<<iter->first<<endl;
-				
+
 			return SYNFIGTOOL_HELP;
 		}
 
@@ -467,16 +468,16 @@ int process_global_flags(arg_list_t &arg_list)
 			synfig::Importer::Book::iterator iter=synfig::Importer::book().begin();
 			for(;iter!=synfig::Importer::book().end();iter++)
 				cout<<iter->first<<endl;
-				
+
 			return SYNFIGTOOL_HELP;
 		}
 
 		if(*iter == "--version")
 		{
 			cerr<<PACKAGE<<" "<<VERSION<<endl;
-		
+
 			arg_list.erase(iter);
-			
+
 			return SYNFIGTOOL_HELP;
 		}
 
@@ -490,18 +491,18 @@ int process_global_flags(arg_list_t &arg_list)
 **	published by the Free Software Foundation; either version 2 of\n\
 **	the License, or (at your option) any later version.\n\
 **\n\
-**	" << endl << endl;	
+**	" << endl << endl;
 			arg_list.erase(iter);
-			
+
 			return SYNFIGTOOL_HELP;
 		}
 
 		if(*iter == "-v")
 		{
 			verbosity++;
-			
+
 			arg_list.erase(iter);
-			
+
 			continue;
 		}
 
@@ -522,14 +523,14 @@ int process_global_flags(arg_list_t &arg_list)
 			continue;
 		}
 	}
-	
+
 	return SYNFIGTOOL_OK;
 }
 
 int extract_arg_cluster(arg_list_t &arg_list,arg_list_t &cluster)
 {
 	arg_list_t::iterator iter, next;
-	
+
 	for(next=arg_list.begin(),iter=next++;iter!=arg_list.end();iter=next++)
 	{
 		if(*iter->begin() != '-')
@@ -565,12 +566,16 @@ int extract_arg_cluster(arg_list_t &arg_list,arg_list_t &cluster)
 			cluster.push_back(*iter);
 			arg_list.erase(iter);
 			iter=next++;
+			if (iter==arg_list.end()) {
+				error("The `%s' flag requires a value.  Use --help for a list of options.", cluster.back().c_str());
+				return SYNFIGTOOL_MISSINGARGUMENT;
+			}
 		}
-			
+
 		cluster.push_back(*iter);
 		arg_list.erase(iter);
 	}
-	
+
 	return SYNFIGTOOL_OK;
 }
 
@@ -588,14 +593,14 @@ int extract_RendDesc(arg_list_t &arg_list,RendDesc &desc)
 			w=atoi(iter->c_str());
 			arg_list.erase(iter);
 		}
-		if(*iter=="-h")
+		else if(*iter=="-h")
 		{
 			arg_list.erase(iter);
 			iter=next++;
 			h=atoi(iter->c_str());
 			arg_list.erase(iter);
 		}
-		if(*iter=="-a")
+		else if(*iter=="-a")
 		{
             int a;
 			arg_list.erase(iter);
@@ -605,7 +610,7 @@ int extract_RendDesc(arg_list_t &arg_list,RendDesc &desc)
 			VERBOSE_OUT(1)<<strprintf(_("Antialiasing set to %d, (%d samples per pixel)"),a,a*a)<<endl;
 			arg_list.erase(iter);
 		}
-		if(*iter=="-s")
+		else if(*iter=="-s")
 		{
 			arg_list.erase(iter);
 			iter=next++;
@@ -613,7 +618,7 @@ int extract_RendDesc(arg_list_t &arg_list,RendDesc &desc)
 			VERBOSE_OUT(1)<<strprintf(_("Span set to %d units"),span)<<endl;
 			arg_list.erase(iter);
 		}
-		if(*iter=="--fps")
+		else if(*iter=="--fps")
 		{
 			arg_list.erase(iter);
 			iter=next++;
@@ -622,7 +627,7 @@ int extract_RendDesc(arg_list_t &arg_list,RendDesc &desc)
 			arg_list.erase(iter);
 			VERBOSE_OUT(1)<<strprintf(_("Frame rate set to %d frames per second"),fps)<<endl;
 		}
-		if(*iter=="--dpi")
+		else if(*iter=="--dpi")
 		{
 			arg_list.erase(iter);
 			iter=next++;
@@ -632,7 +637,7 @@ int extract_RendDesc(arg_list_t &arg_list,RendDesc &desc)
 			arg_list.erase(iter);
 			VERBOSE_OUT(1)<<strprintf(_("Physical resolution set to %f dpi"),dpi)<<endl;
 		}
-		if(*iter=="--dpi-x")
+		else if(*iter=="--dpi-x")
 		{
 			arg_list.erase(iter);
 			iter=next++;
@@ -642,7 +647,7 @@ int extract_RendDesc(arg_list_t &arg_list,RendDesc &desc)
 			arg_list.erase(iter);
 			VERBOSE_OUT(1)<<strprintf(_("Physical X resolution set to %f dpi"),dpi)<<endl;
 		}
-		if(*iter=="--dpi-y")
+		else if(*iter=="--dpi-y")
 		{
 			arg_list.erase(iter);
 			iter=next++;
@@ -652,21 +657,21 @@ int extract_RendDesc(arg_list_t &arg_list,RendDesc &desc)
 			arg_list.erase(iter);
 			VERBOSE_OUT(1)<<strprintf(_("Physical Y resolution set to %f dpi"),dpi)<<endl;
 		}
-		if(*iter=="--start-time" || *iter=="--begin-time")
+		else if(*iter=="--start-time" || *iter=="--begin-time")
 		{
 			arg_list.erase(iter);
 			iter=next++;
 			desc.set_time_start(Time(*iter,desc.get_frame_rate()));
 			arg_list.erase(iter);
 		}
-		if(*iter=="--end-time")
+		else if(*iter=="--end-time")
 		{
 			arg_list.erase(iter);
 			iter=next++;
 			desc.set_time_end(Time(*iter,desc.get_frame_rate()));
 			arg_list.erase(iter);
 		}
-		if(*iter=="--time")
+		else if(*iter=="--time")
 		{
 			arg_list.erase(iter);
 			iter=next++;
@@ -674,13 +679,13 @@ int extract_RendDesc(arg_list_t &arg_list,RendDesc &desc)
 			VERBOSE_OUT(1)<<_("Rendering frame at ")<<desc.get_time_start().get_string(desc.get_frame_rate())<<endl;
 			arg_list.erase(iter);
 		}
-		if(*iter=="-g")
+		else if(*iter=="-g")
 		{
 			synfig::warning("Gamma argument is currently ignored");
-			//arg_list.erase(iter);
-			//iter=next++;
+			arg_list.erase(iter);
+			iter=next++;
 			//desc.set_gamma(Gamma(atof(iter->c_str())));
-			//arg_list.erase(iter);
+			arg_list.erase(iter);
 		}
 	}
 	if(w&&h)
@@ -720,7 +725,7 @@ int extract_quality(arg_list_t &arg_list,int &quality)
 			arg_list.erase(iter);
 		}
 	}
-	
+
 	return SYNFIGTOOL_OK;
 }
 
@@ -738,7 +743,7 @@ int extract_threads(arg_list_t &arg_list,int &threads)
 			arg_list.erase(iter);
 		}
 	}
-	
+
 	return SYNFIGTOOL_OK;
 }
 
@@ -746,7 +751,7 @@ int extract_target(arg_list_t &arg_list,string &type)
 {
 	arg_list_t::iterator iter, next;
 	type.clear();
-	
+
 	for(next=arg_list.begin(),iter=next++;iter!=arg_list.end();iter=next++)
 	{
 		if(*iter=="-t")
@@ -765,7 +770,7 @@ int extract_append(arg_list_t &arg_list,string &filename)
 {
 	arg_list_t::iterator iter, next;
 	filename.clear();
-	
+
 	for(next=arg_list.begin(),iter=next++;iter!=arg_list.end();iter=next++)
 	{
 		if(*iter=="--append")
@@ -785,7 +790,7 @@ int extract_outfile(arg_list_t &arg_list,string &outfile)
 	arg_list_t::iterator iter, next;
 	int ret=SYNFIGTOOL_FILENOTFOUND;
 	outfile.clear();
-	
+
 	for(next=arg_list.begin(),iter=next++;iter!=arg_list.end();iter=next++)
 	{
 		if(*iter=="-o")
@@ -805,7 +810,7 @@ int extract_canvasid(arg_list_t &arg_list,string &canvasid)
 {
 	arg_list_t::iterator iter, next;
 	//canvasid.clear();
-	
+
 	for(next=arg_list.begin(),iter=next++;iter!=arg_list.end();iter=next++)
 	{
 		if(*iter=="-c")
@@ -829,10 +834,10 @@ int main(int argc, char *argv[])
 	int i;
 	arg_list_t arg_list;
 	job_list_t job_list;
-	
+
 	progname=argv[0];
 	Progress p(argv[0]);
-	
+
 	if(!SYNFIG_CHECK_VERSION())
 	{
 		cerr<<_("FATAL: Synfig Version Mismatch")<<endl;
@@ -846,31 +851,34 @@ int main(int argc, char *argv[])
 
 	for(i=1;i<argc;i++)
 		arg_list.push_back(argv[i]);
-	
+
 	if((i=process_global_flags(arg_list)))
 		return i;
 
 	VERBOSE_OUT(1)<<_("verbosity set to ")<<verbosity<<endl;
 	synfig::Main synfig_main(dirname(progname),&p);
-	
+
 	{
 		arg_list_t defaults, imageargs;
-		
+		int ret;
+
 		// Grab the defaults before the first file
-		extract_arg_cluster(arg_list,defaults);
-		
+		if ((ret = extract_arg_cluster(arg_list,defaults)) != SYNFIGTOOL_OK)
+		  return ret;
+
 		while(arg_list.size())
 		{
 			string target_name;
 			job_list.push_front(Job());
 			int threads=0;
-			
+
 			imageargs=defaults;
 			job_list.front().filename=arg_list.front();
 			arg_list.pop_front();
 
-			extract_arg_cluster(arg_list,imageargs);			
-			
+			if ((ret = extract_arg_cluster(arg_list,imageargs)) != SYNFIGTOOL_OK)
+			  return ret;
+
 			// Open the composition
 			{
 				job_list.front().root=open_canvas(job_list.front().filename);
@@ -884,7 +892,7 @@ int main(int argc, char *argv[])
 			}
 
 			job_list.front().root->set_time(0);
-			
+
 			string canvasid;
 			extract_canvasid(imageargs,canvasid);
 			if(!canvasid.empty())
@@ -899,7 +907,7 @@ int main(int argc, char *argv[])
 					cerr<<_("Throwing out job...")<<endl;
 					job_list.pop_front();
 					continue;
-					
+
 				}
 				catch(Exception::BadLinkName)
 				{
@@ -907,12 +915,12 @@ int main(int argc, char *argv[])
 					cerr<<_("Throwing out job...")<<endl;
 					job_list.pop_front();
 					continue;
-					
+
 				}
 			}
 			else
 				job_list.front().canvas=job_list.front().root;
-			
+
 			extract_RendDesc(imageargs,job_list.front().canvas->rend_desc());
 			extract_target(imageargs,target_name);
 			extract_threads(imageargs,threads);
@@ -941,25 +949,20 @@ int main(int argc, char *argv[])
 					VERBOSE_OUT(2)<<_("Appended contents of ")<<composite_file<<endl;
 				}
 			}while(false);
-			
+
 			VERBOSE_OUT(4)<<_("Attempting to determine target/outfile...")<<endl;
-			
+
 			// If the target type is not yet defined,
 			// try to figure it out from the outfile.
 			if(target_name.empty() && !job_list.front().outfilename.empty())
-			try
 			{
 				VERBOSE_OUT(3)<<_("Target name undefined, attempting to figure it out")<<endl;
-				string ext=string(find(job_list.front().outfilename.begin(),job_list.front().outfilename.end(),'.')+1,job_list.front().outfilename.end());
+				string ext=job_list.front().outfilename.substr(job_list.front().outfilename.rfind('.')+1);
 				if(Target::ext_book().count(ext))
 					target_name=Target::ext_book()[ext];
 				else
 					target_name=ext;
 			}
-			catch(std::length_error)
-			{
-				synfig::warning("Length error caught when attempting to figure out target name");
-			}
 
 			// If the target type is STILL not yet defined, then
 			// set it to a some sort of default
@@ -985,10 +988,10 @@ int main(int argc, char *argv[])
 
 			VERBOSE_OUT(4)<<"target_name="<<target_name<<endl;
 			VERBOSE_OUT(4)<<"outfile_name="<<job_list.front().outfilename<<endl;
-			
+
 			VERBOSE_OUT(4)<<_("Creating the target...")<<endl;
 			job_list.front().target=synfig::Target::create(target_name,job_list.front().outfilename);
-			
+
 			if(target_name=="sif")
 			{
 				job_list.front().sifout=true;
@@ -1010,16 +1013,16 @@ int main(int argc, char *argv[])
 			{
 				VERBOSE_OUT(4)<<_("Setting the canvas on the target...")<<endl;
 				job_list.front().target->set_canvas(job_list.front().canvas);
-				VERBOSE_OUT(4)<<_("Setting the quality of the target...")<<endl;				
+				VERBOSE_OUT(4)<<_("Setting the quality of the target...")<<endl;
 				job_list.front().target->set_quality(job_list.front().quality);
 			}
-			
+
 			// Set the threads for the target
 			if(job_list.front().target && Target_Scanline::Handle::cast_dynamic(job_list.front().target))
 			{
 				Target_Scanline::Handle::cast_dynamic(job_list.front().target)->set_threads(threads);
 			}
-			
+
 			if(imageargs.size())
 			{
 				cerr<<_("Unidentified arguments for ")<<job_list.front().filename<<": ";
@@ -1034,7 +1037,7 @@ int main(int argc, char *argv[])
 			//getline(cin,bleh);
 		}
 	}
-	
+
 	if(arg_list.size())
 	{
 		cerr<<_("Unidentified arguments:");
@@ -1043,13 +1046,13 @@ int main(int argc, char *argv[])
 		cerr<<endl;
 		return SYNFIGTOOL_UNKNOWNARGUMENT;
 	}
-	
+
 	if(!job_list.size())
 	{
 		cerr<<_("Nothing to do!")<<endl;
 		return SYNFIGTOOL_BORRED;
 	}
-	
+
 	for(;job_list.size();job_list.pop_front())
 	{
 		VERBOSE_OUT(3)<<job_list.front().filename<<" -- "<<endl<<'\t'<<
@@ -1066,7 +1069,7 @@ int main(int argc, char *argv[])
 			job_list.front().desc.get_br()[0],job_list.front().desc.get_br()[1],
 			job_list.front().desc.get_focus()[0],job_list.front().desc.get_focus()[1]
 			)<<endl;
-		
+
 		RenderProgress p;
 		p.task(job_list.front().filename+" ==> "+job_list.front().outfilename);
 		if(!job_list.front().sifout)
@@ -1094,10 +1097,10 @@ int main(int argc, char *argv[])
 			}
 		}
 	}
-	
+
 	job_list.clear();
-	
+
 	VERBOSE_OUT(1)<<_("Done.")<<endl;
-	
+
 	return SYNFIGTOOL_OK;
 }
